@@ -42,23 +42,16 @@ typedef enum
 	NOTIF_NICK
 } NotifStatus;
 
-/* Represents a channel entry in the pop-up menu. */
-struct MenuChannel
-{
-	NotifStatus	status;
-	GtkWidget*	menu_item;
-};
-
-static xchat_plugin*		ph;			/* Plugin handle. */
-static xchat_gnome_plugin*	xgph;			/* xchat gnome plugin handle. */
-static NotifStatus		status = NOTIF_NONE;	/* Current status level. */
-static gboolean			focused = TRUE;		/* GTK_WIDGET_HAS_FOCUS doesn't seem to be working... */
-static gboolean			persistant;		/* Keep the icon in the tray at all times? */
-static GtkWidget*		main_window;		/* xchat-gnome's main window. */
-static EggTrayIcon*		notification;		/* Notification area icon. */
-static GtkWidget*		image;			/* The image displayed by the icon. */
-static GdkPixbuf*		pixbufs[4];		/* Pixbufs */
-
+static EggTrayIcon*        notification;        /* Notification area icon. */
+static gboolean            focused = TRUE;      /* GTK_WIDGET_HAS_FOCUS doesn't seem to be working... */
+static gboolean            persistant;          /* Keep the icon in the tray at all times? */
+static gboolean            hidden = FALSE;      /* True when the main window is hidden. */
+static GdkPixbuf*          pixbufs[4];          /* Pixbufs */
+static GtkWidget*          image;               /* The image displayed by the icon. */
+static GtkWidget*          main_window;         /* xchat-gnome's main window. */
+static NotifStatus         status = NOTIF_NONE; /* Current status level. */
+static xchat_gnome_plugin* xgph;                /* xchat gnome plugin handle. */
+static xchat_plugin*       ph;                  /* Plugin handle. */
 
 /*** Callbacks ***/
 static gboolean
@@ -92,7 +85,7 @@ lost_focus_cb (GtkWidget * widget, GdkEventFocus * event, gpointer data)
 static int
 new_msg_cb (char **word, void *msg_lvl)
 {
-	if (status < (NotifStatus) msg_lvl && !focused) {
+	if (status <= (NotifStatus) msg_lvl && !focused) {
 		status = (NotifStatus) msg_lvl;
 		gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbufs[status]);
 		gtk_widget_show_all (GTK_WIDGET (notification));
@@ -101,35 +94,24 @@ new_msg_cb (char **word, void *msg_lvl)
 	return 0;
 }
 
-static int
-join_chan_cb (char **word, void *data)
-{
-	return 0;
-}
-
-static int
-part_chan_cb (char **word, void *data)
-{
-	return 0;
-}
-
-static int
-chan_changed_cb (char **word, void *data)
-{
-	return 0;
-}
-
 static gboolean
 notification_clicked_cb (GtkWidget * widget, GdkEventButton * event, gpointer data)
 {
 	switch (event->button) {
 		/* Left click. */
 		case 1:
-			if (gdk_window_get_state (main_window->window) & GDK_WINDOW_STATE_ICONIFIED) {
+			if (persistant) {
+				if (hidden) {
+					xchat_command (ph, "GUI SHOW");
+				} else {
+					xchat_command (ph, "GUI HIDE");
+				}
+
+				hidden = !hidden;
+			} else {
 				gtk_window_present (GTK_WINDOW (main_window));
-			} else if (persistant) {
-				xchat_command (ph, "GUI HIDE");
 			}
+
 			break;
 
 		default:
@@ -242,9 +224,6 @@ xchat_plugin_init (xchat_plugin * plugin_handle, char **plugin_name, char **plug
 	xchat_hook_print (ph, "Channel Msg Hilight",		XCHAT_PRI_NORM, new_msg_cb, (gpointer) NOTIF_NICK);
 	xchat_hook_print (ph, "Channel Action Hilight",		XCHAT_PRI_NORM, new_msg_cb, (gpointer) NOTIF_NICK);
 	xchat_hook_print (ph, "Private Message to Dialog",	XCHAT_PRI_NORM, new_msg_cb, (gpointer) NOTIF_MSG);
-	xchat_hook_print (ph, "You Join",			XCHAT_PRI_NORM, join_chan_cb, NULL);
-	xchat_hook_print (ph, "You Part",			XCHAT_PRI_NORM, part_chan_cb, NULL);
-	xchat_hook_print (ph, "Focus Tab",			XCHAT_PRI_NORM, chan_changed_cb, NULL);
 
 	xchat_print (ph, "Notification plugin loaded.\n");
 
