@@ -28,7 +28,12 @@ void fe_timeout_remove (int tag);
 void fe_new_window (struct session *sess, int focus);
 void fe_new_server (struct server *serv);
 void fe_add_rawlog (struct server *serv, char *text, int len, int outbound);
-void fe_message (char *msg, int wait);
+#define FE_MSG_WAIT 1
+#define FE_MSG_INFO 2
+#define FE_MSG_WARN 4
+#define FE_MSG_ERROR 8
+#define FE_MSG_HASTITLE 16
+void fe_message (char *msg, int flags);
 #define FIA_READ 1
 #define FIA_WRITE 2
 #define FIA_EX 4
@@ -61,6 +66,7 @@ void fe_userlist_rehash (struct session *sess, struct User *user);
 void fe_userlist_move (struct session *sess, struct User *user, int new_row);
 void fe_userlist_numbers (struct session *sess);
 void fe_userlist_clear (struct session *sess);
+void fe_uselect (session *sess, char *word[], int do_clear, int scroll_to);
 void fe_dcc_add (struct DCC *dcc);
 void fe_dcc_update (struct DCC *dcc);
 void fe_dcc_remove (struct DCC *dcc);
@@ -90,6 +96,7 @@ void fe_get_str (char *prompt, char *def, void *callback, void *ud);
 void fe_get_int (char *prompt, int def, void *callback, void *ud);
 void fe_ctrl_gui (session *sess, int action, int arg);
 int fe_gui_info (session *sess, int info_type);
+void *fe_gui_info_ptr (session *sess, int info_type);
 void fe_confirm (const char *message, void (*yesproc)(void *), void (*noproc)(void *), void *ud);
 char *fe_get_inputbox_contents (struct session *sess);
 int fe_get_inputbox_cursor (struct session *sess);
@@ -99,6 +106,10 @@ void fe_open_url (const char *url);
 void fe_menu_del (menu_entry *);
 void fe_menu_add (menu_entry *);
 void fe_menu_update (menu_entry *);
-void fe_uselect (session *sess, char *word[], int do_clear, int scroll_to);
+#define FE_SE_CONNECT 0
+#define FE_SE_LOGGEDIN 1
+#define FE_SE_DISCONNECT 2
+#define FE_SE_RECONDELAY 3
+void fe_server_event (server *serv, int type, int arg);
 
 #endif
