@@ -28,9 +28,17 @@
 typedef struct
 {
 	GtkListStore *store;
+	GtkTreeModel *filter;
 	GtkTreeModelSort *sort;
 	GladeXML *xml;
 	struct server *server;
+
+	int minimum, maximum;
+	char *text_filter;
+	gboolean filter_topic, filter_name;
+	guint refresh_timeout;
+	guint refresh_calls;
+	gboolean empty;
 } channel_list_window;
 
 gboolean channel_list_exists (server *serv);
