@@ -82,7 +82,7 @@ fe_args (int argc, char *argv[])
 	if (opt_cfgdir)
 		xdir_fs = opt_cfgdir;
 
-	gnome_program_init ("xchat-gnome", VERSION, LIBGNOMEUI_MODULE, argc, argv, NULL);
+	gnome_program_init ("xchat-gnome", VERSION, LIBGNOMEUI_MODULE, argc, argv, GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
 
 	/* FIXME: this is kind of a silly place to put this, but it seems to want to
 	   follow gnome_program_init */
@@ -111,6 +111,7 @@ fe_init (void)
 	/* Force various window-related options to match our interaction model */
 	prefs.use_server_tab = TRUE;
 	prefs.notices_tabs = FALSE;
+	prefs.servernotice = TRUE;
 
 	/* If we don't have a specific DCC IP address, force get-from-server */
 	if (strlen (prefs.dcc_ip_str) == 0)
@@ -121,9 +122,9 @@ fe_init (void)
 	 */
 	arg_skip_plugins = 1;
 
+#ifdef USE_PLUGIN
 	plugins_initialize ();
-	if (!opt_noplugins)
-		autoload_plugins ();
+#endif
 }
 
 void
@@ -167,11 +168,19 @@ fe_timeout_remove (int tag)
 void
 fe_new_window (struct session *sess, int focus)
 {
+	static gboolean loaded = FALSE;
+
 	text_gui_add_text_buffer (sess);
 	if (sess->type == SESS_SERVER)
 		navigation_tree_create_new_network_entry (gui.server_tree, sess);
 	else if (sess->type == SESS_CHANNEL || sess->type == SESS_DIALOG)
 		navigation_tree_create_new_channel_entry (gui.server_tree, sess);
+#ifdef USE_PLUGIN
+	if (!(opt_noplugins || loaded)) {
+		loaded = TRUE;
+		autoload_plugins ();
+	}
+#endif
 }
 
 void
