@@ -20,15 +20,13 @@
  */
 
 #include "gui.h"
-#include "main_window.h"
-#include "preferences_dialog.h"
-#include "connect_dialog.h"
-#include "navigation_tree.h"
+#include "main-window.h"
+#include "preferences-dialog.h"
+#include "connect-dialog.h"
 #include "about.h"
 #include "textgui.h"
-#include "userlist_gui.h"
+#include "userlist-gui.h"
 #include "pixmaps.h"
-#include "transfers.h"
 #include "../common/text.h"
 
 XChatGUI gui;
@@ -55,10 +53,7 @@ initialize_gui_2 ()
 	pixmaps_init ();
 	initialize_main_window ();
 	initialize_text_gui ();
-	initialize_preferences_dialog ();
-	initialize_connection_dialog ();
 	initialize_userlist ();
-	initialize_transfers_window ();
 
 	gui.tree_model = navigation_model_new ();
 	gui.server_tree = navigation_tree_new (gui.tree_model);
@@ -66,6 +61,8 @@ initialize_gui_2 ()
 	gtk_widget_show (GTK_WIDGET (gui.server_tree));
 	gtk_container_add (GTK_CONTAINER (widget), GTK_WIDGET (gui.server_tree));
 
+	gui.dcc = dcc_window_new ();
+
 	return TRUE;
 }
 
