@@ -1,7 +1,7 @@
 /*
  * irc-network-editor.h - GtkDialog subclass for editing an IrcNetwork
  *
- * Copyright (C) 2005 David Trowbridge
+ * Copyright (C) 2004-2005 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -287,7 +287,6 @@ irc_network_editor_init (IrcNetworkEditor *dialog)
 		enc++;
 	} while (*enc);
 
-	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_APPLY,  GTK_RESPONSE_APPLY);
 	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_OK,     GTK_RESPONSE_OK);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
@@ -366,7 +365,7 @@ populate_autojoin_list (IrcNetworkEditor *e)
 	channels = g_strsplit (e->network->autojoin, ",", 0);
 	for (i = 0; channels[i]; i++) {
 		gtk_list_store_append (e->autojoin_store, &iter);
-		gtk_list_store_set (e->autojoin_store, &iter, 0, channels[i]);
+		gtk_list_store_set (e->autojoin_store, &iter, 0, channels[i], -1);
 	}
 
 	g_strfreev (channels);
@@ -470,14 +469,15 @@ apply_changes (IrcNetworkEditor *e)
 		g_free (serv->hostname);
 		g_free (serv);
 	}
-	g_slist_free (s);
-	s = NULL;
+	g_slist_free (net->servers);
+	net->servers = NULL;
 	if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (e->server_store), &iter)) {
 		do {
 			char *text;
 			ircserver *serv = g_new0 (ircserver, 1);
 			gtk_tree_model_get (GTK_TREE_MODEL (e->server_store), &iter, 0, &text, -1);
 			serv->hostname = g_strdup (text);
+			net->servers = g_slist_append (net->servers, serv);
 		} while (gtk_tree_model_iter_next (GTK_TREE_MODEL (e->server_store), &iter));
 	}
 
@@ -536,11 +536,6 @@ irc_network_editor_run (IrcNetworkEditor *editor)
 	gint response;
 
 	response = gtk_dialog_run (GTK_DIALOG (editor));
-	while (response == GTK_RESPONSE_APPLY) {
-		if (check_input (editor))
-			apply_changes (editor);
-		response = gtk_dialog_run (GTK_DIALOG (editor));
-	}
 	while (response == GTK_RESPONSE_OK) {
 		if (check_input (editor)) {
 			apply_changes (editor);
