@@ -56,8 +56,6 @@ static void *last_search_position = NULL;
 
 static void on_main_window_close (GtkWidget *widget, GdkEvent *event, gpointer data);
 static void on_discussion_jump_activate (GtkAccelGroup *accelgroup, GObject *arg1, guint arg2, GdkModifierType arg3, gpointer data);
-static void on_discussion_plus_activate (GtkAccelGroup *accelgroup, GObject *arg1, guint arg2, GdkModifierType arg3, gpointer data);
-static void on_discussion_minus_activate (GtkAccelGroup *accelgroup, GObject *arg1, guint arg2, GdkModifierType arg3, gpointer data);
 static void on_pgup (GtkAccelGroup *accelgroup, GObject *arg1, guint arg2, GdkModifierType arg3, gpointer data);
 static void on_pgdn (GtkAccelGroup *accelgroup, GObject *arg1, guint arg2, GdkModifierType arg3, gpointer data);
 
@@ -70,7 +68,6 @@ static void on_edit_copy_activate (GtkAction *action, gpointer data);
 static void on_edit_paste_activate (GtkAction *action, gpointer data);
 static void on_edit_clear_activate (GtkAction *action, gpointer data);
 static void on_edit_preferences_activate (GtkAction *action, gpointer data);
-static void on_insert_color_code_activate (GtkAction *action, gpointer data);
 static void on_network_information_activate (GtkAction *action, gpointer data);
 static void on_network_reconnect_activate (GtkAction *action, gpointer data);
 static void on_network_disconnect_activate (GtkAction *action, gpointer data);
@@ -81,7 +78,6 @@ static void on_discussion_save_activate (GtkAction *action, gpointer data);
 static void on_discussion_leave_activate (GtkAction *action, gpointer data);
 static void on_discussion_close_activate (GtkAction *action, gpointer data);
 static void on_discussion_find_activate (GtkAction *action, gpointer data);
-static void on_discussion_find_next_activate (GtkAction *action, gpointer data);
 static void on_discussion_clear_window_activate (GtkAction *action, gpointer data);
 static void on_discussion_bans_activate (GtkAction *action, gpointer data);
 static void on_discussion_topic_change_activate (GtkButton *widget, gpointer data);
@@ -89,13 +85,13 @@ static void on_go_previous_network_activate (GtkAction *action, gpointer data);
 static void on_go_next_network_activate (GtkAction *action, gpointer data);
 static void on_go_previous_discussion_activate (GtkAction *action, gpointer data);
 static void on_go_next_discussion_activate (GtkAction *action, gpointer data);
+static void on_help_contents_activate (GtkAction *action, gpointer data);
 static void on_help_about_activate (GtkAction *action, gpointer data);
+static void on_nickname_clicked (GtkButton *widget, gpointer user_data);
 
 static void on_add_widget (GtkUIManager *manager, GtkWidget *menu, GtkWidget *menu_vbox);
 
-#if (GTK_CHECK_VERSION(2,5,0))
 static void on_expand_topic (GtkExpander *expander, gpointer data);
-#endif
 
 static void on_text_entry_activate (GtkWidget *widget, gpointer data);
 static gboolean on_text_entry_key (GtkWidget *widget, GdkEventKey *key, gpointer data);
@@ -109,33 +105,26 @@ static void entry_context (GtkEntry *entry, GtkMenu *menu, gpointer user_data);
 static GtkActionEntry action_entries [] = {
 
 	/* Toplevel */
-	{ "IRC", NULL, _("_IRC") },
-	{ "Edit", NULL, _("_Edit") },
-	{ "Insert", NULL, _("In_sert") },
-	{ "Network", NULL, _("_Network") },
-	{ "Discussion", NULL, _("_Discussion") },
-	{ "Go", NULL, _("_Go") },
-	{ "Help", NULL, _("_Help") },
+	{ "IRC",         NULL, _("_IRC") },
+	{ "Edit",        NULL, _("_Edit") },
+	{ "Insert",      NULL, _("In_sert") },
+	{ "Network",     NULL, _("_Network") },
+	{ "Discussion",  NULL, _("_Discussion") },
+	{ "Go",          NULL, _("_Go") },
+	{ "Help",        NULL, _("_Help") },
 	{ "PopupAction", NULL, _("") },
 
 	/* IRC menu */
-#if (GTK_CHECK_VERSION(2,5,0))
-	{ "IRCConnect", GTK_STOCK_CONNECT, _("_Connect"), "<control>N", NULL, G_CALLBACK (on_irc_connect_activate) },
-#else
-	{ "IRCConnect", NULL, _("_Connect"), "<control>N", NULL, G_CALLBACK (on_irc_connect_activate) },
-#endif
-	{ "IRCDownloads", NULL, _("_File Transfers"), "<alt>F", NULL, G_CALLBACK (on_irc_downloads_activate) },
-	{ "IRCQuit", GTK_STOCK_QUIT, _("_Quit"), "<control>Q", NULL, G_CALLBACK (on_irc_quit_activate) },
+	{ "IRCConnect",   GTK_STOCK_CONNECT, _("_Connect"),        "<control>N", NULL, G_CALLBACK (on_irc_connect_activate) },
+	{ "IRCDownloads", NULL,              _("_File Transfers"), "<alt>F",     NULL, G_CALLBACK (on_irc_downloads_activate) },
+	{ "IRCQuit",      GTK_STOCK_QUIT,    _("_Quit"),           "<control>Q", NULL, G_CALLBACK (on_irc_quit_activate) },
 
 	/* Edit menu */
-	{ "EditCut", GTK_STOCK_CUT, _("Cu_t"), "<control>X", NULL, G_CALLBACK (on_edit_cut_activate) },
-	{ "EditCopy", GTK_STOCK_COPY, _("_Copy"), "<control>C", NULL, G_CALLBACK (on_edit_copy_activate) },
-	{ "EditPaste", GTK_STOCK_PASTE, _("_Paste"), "<control>V", NULL, G_CALLBACK (on_edit_paste_activate) },
-	{ "EditClear", GTK_STOCK_CLEAR, _("C_lear"), "", NULL, G_CALLBACK (on_edit_clear_activate) },
-	{ "EditPreferences", GTK_STOCK_PREFERENCES, _("Prefere_nces"), "", NULL, G_CALLBACK (on_edit_preferences_activate) },
-
-	/* Insert menu */
-	{ "InsertColorCode", GTK_STOCK_SELECT_COLOR, _("Color Code..."), "", NULL, G_CALLBACK (on_insert_color_code_activate) },
+	{ "EditCut",         GTK_STOCK_CUT,         _("Cu_t"),         "<control>X", NULL, G_CALLBACK (on_edit_cut_activate) },
+	{ "EditCopy",        GTK_STOCK_COPY,        _("_Copy"),        "<control>C", NULL, G_CALLBACK (on_edit_copy_activate) },
+	{ "EditPaste",       GTK_STOCK_PASTE,       _("_Paste"),       "<control>V", NULL, G_CALLBACK (on_edit_paste_activate) },
+	{ "EditClear",       GTK_STOCK_CLEAR,       _("C_lear"),       "",           NULL, G_CALLBACK (on_edit_clear_activate) },
+	{ "EditPreferences", GTK_STOCK_PREFERENCES, _("Prefere_nces"), "",           NULL, G_CALLBACK (on_edit_preferences_activate) },
 
 	/* Network menu */
 	{ "NetworkInformation", GTK_STOCK_DIALOG_INFO, _("_Information"), "", NULL, G_CALLBACK (on_network_information_activate) },
@@ -150,7 +139,6 @@ static GtkActionEntry action_entries [] = {
 	{ "DiscussionLeave", GTK_STOCK_QUIT, _("_Leave"), "", NULL, G_CALLBACK (on_discussion_leave_activate) },
 	{ "DiscussionClose", GTK_STOCK_CLOSE, _("Cl_ose"), "<control>W", NULL, G_CALLBACK (on_discussion_close_activate) },
 	{ "DiscussionFind", GTK_STOCK_FIND, _("_Find"), "<control>F", NULL, G_CALLBACK (on_discussion_find_activate) },
-	{ "DiscussionFindNext", NULL, _("Find Ne_xt"), "<control>G", NULL, G_CALLBACK (on_discussion_find_next_activate) },
 	{ "DiscussionClearWindow", GTK_STOCK_CLEAR, _("_Clear Window"), "<control>L", NULL, G_CALLBACK (on_discussion_clear_window_activate) },
 	{ "DiscussionChangeTopic", GTK_STOCK_REFRESH, _("Change _Topic"), "<alt>T", NULL, G_CALLBACK (on_discussion_topic_change_activate) },
 	{ "DiscussionBans", GTK_STOCK_DIALOG_WARNING, _("_Bans"), "<alt>B", NULL, G_CALLBACK (on_discussion_bans_activate) },
@@ -162,16 +150,12 @@ static GtkActionEntry action_entries [] = {
 	{ "GoNextDiscussion", GTK_STOCK_GO_FORWARD, _("_Next Discussion"), "<alt>Down", NULL, G_CALLBACK (on_go_next_discussion_activate) },
 
 	/* Help menu */
-#if (GTK_CHECK_VERSION(2,5,0))
-	{ "HelpAbout", GTK_STOCK_ABOUT, _("_About"), "", NULL, G_CALLBACK (on_help_about_activate) },
-#else
-	{ "HelpAbout", NULL, _("_About"), "", NULL, G_CALLBACK (on_help_about_activate) },
-#endif
+	{ "HelpContents", GTK_STOCK_HELP,  _("_Contents"), "",   NULL, G_CALLBACK (on_help_contents_activate) },
+	{ "HelpAbout",    GTK_STOCK_ABOUT, _("_About"),    "",   NULL, G_CALLBACK (on_help_about_activate) },
 };
 
-/*
- * FIXME: for the love of god make this more efficient
- */
+static GCompletion *command_completion;
+
 GtkActionEntry *
 find_action_entry (gchar *name)
 {
@@ -204,6 +188,49 @@ find_previous_action (gchar *name)
 	return NULL;
 }
 
+void
+save_transcript ()
+{
+	GtkWidget *file_chooser;
+
+	file_chooser = gtk_file_chooser_dialog_new ("Save Transcript",
+	                                            GTK_WINDOW (gui.main_window),
+	                                            GTK_FILE_CHOOSER_ACTION_SAVE,
+	                                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+	                                            GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+	                                            NULL);
+	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (file_chooser), TRUE);
+	gtk_dialog_set_default_response (GTK_DIALOG (file_chooser), GTK_RESPONSE_ACCEPT);
+
+	if (gtk_dialog_run (GTK_DIALOG (file_chooser)) == GTK_RESPONSE_ACCEPT) {
+		gchar *filename;
+		GIOChannel *file;
+		GError *error = NULL;
+
+		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_chooser));
+		file = g_io_channel_new_file (filename, "w", &error);
+		if (error) {
+			gchar *header = g_strdup_printf ("Error saving %s", filename);
+			error_dialog (header, error->message);
+			g_free (header);
+			g_error_free (error);
+		} else {
+			gint fd = g_io_channel_unix_get_fd (file);
+			gtk_xtext_save (gui.xtext, fd);
+			g_io_channel_shutdown (file, TRUE, &error);
+
+			if (error) {
+				gchar *header = g_strdup_printf ("Error saving %s", filename);
+				error_dialog (header, error->message);
+				g_free (header);
+				g_error_free (error);
+			}
+		}
+		g_free (filename);
+	}
+	gtk_widget_destroy (file_chooser);
+}
+
 static void
 keybinding_key_changed (GConfClient *client, guint cnxn_id, GConfEntry *e, gpointer user_data)
 {
@@ -278,6 +305,7 @@ setup_menu_item (GConfClient *client, GtkActionEntry *entry)
 	if (e == NULL) {
 		/* no GConf data, so let's use the hardcoded values */
 		gtk_action_group_add_actions (gui.action_group, entry, 1, NULL);
+		g_free (key_string);
 		return;
 	}
 
@@ -292,6 +320,7 @@ setup_menu_item (GConfClient *client, GtkActionEntry *entry)
 	if (entry->callback != NULL)
 		g_signal_connect (action, "activate", G_CALLBACK (entry->callback), gui.xml);
 
+	g_object_unref (action);
 	g_free (key_string);
 	gconf_entry_free (e);
 }
@@ -317,6 +346,9 @@ close_find_button (GtkWidget *button, gpointer data)
 	widget = glade_xml_get_widget (gui.xml, "find hbox");
 	gtk_widget_hide (widget);
 	gtk_widget_grab_focus (gui.text_entry);
+
+	widget = glade_xml_get_widget (gui.xml, "find_status_label");
+	gtk_label_set_text (GTK_LABEL (widget), "");
 }
 
 static gboolean
@@ -331,8 +363,29 @@ close_find_key (GtkWidget *entry, GdkEventKey *event, gpointer data)
 static void
 find_next (GtkWidget *entry, gpointer data)
 {
+	GtkWidget *info;
 	const guchar *text = gtk_entry_get_text (GTK_ENTRY (entry));
 	last_search_position = gtk_xtext_search (GTK_XTEXT (gui.xtext), text, last_search_position);
+
+	info = glade_xml_get_widget (gui.xml, "find_status_label");
+	if (last_search_position == NULL)
+		gtk_label_set_markup (GTK_LABEL (info), "<span foreground=\"grey\">Search string not found</span>");
+	else
+		gtk_label_set_text (GTK_LABEL (info), "");
+}
+
+static void
+find_button_next (GtkButton *button, GtkWidget *entry)
+{
+	GtkWidget *info;
+	const guchar *text = gtk_entry_get_text (GTK_ENTRY (entry));
+	last_search_position = gtk_xtext_search (GTK_XTEXT (gui.xtext), text, last_search_position);
+
+	info = glade_xml_get_widget (gui.xml, "find_status_label");
+	if (last_search_position == NULL)
+		gtk_label_set_markup (GTK_LABEL (info), "<span foreground=\"grey\">Search string not found</span>");
+	else
+		gtk_label_set_text (GTK_LABEL (info), "");
 }
 
 static void
@@ -376,8 +429,10 @@ url_activated (GtkWidget *url_label, const char *url, gpointer data)
 void
 initialize_main_window ()
 {
-	GtkWidget *entrybox, *topicbox, *close, *menu_vbox, *widget;
+	GtkWidget *entrybox, *topicbox, *close, *menu_vbox, *widget, *widget2;
 	GError *error = NULL;
+	GList *tmp = NULL;
+	int i;
 
 	gui.main_window = GNOME_APP (glade_xml_get_widget (gui.xml, "xchat-gnome"));
 	g_signal_connect (G_OBJECT (gui.main_window), "delete-event",
@@ -440,7 +495,7 @@ initialize_main_window ()
 	gtk_box_pack_start (GTK_BOX (topicbox), gui.topic_label, TRUE, TRUE, 0);
 	gtk_box_reorder_child (GTK_BOX (topicbox), gui.topic_label, 1);
 	gtk_label_set_selectable (GTK_LABEL (gui.topic_label), TRUE);
-#if (GTK_CHECK_VERSION(2,5,0))
+
 	gui.topic_expander = GTK_EXPANDER (gtk_expander_new (NULL));
 	gtk_widget_show (GTK_WIDGET (gui.topic_expander));
 	gtk_box_pack_start (GTK_BOX (topicbox), GTK_WIDGET (gui.topic_expander), FALSE, TRUE, 0);
@@ -449,9 +504,6 @@ initialize_main_window ()
 	gtk_expander_set_use_markup (gui.topic_expander, TRUE);
 	g_signal_connect (G_OBJECT (gui.topic_expander), "activate", G_CALLBACK (on_expand_topic), NULL);
 	gtk_label_set_ellipsize (GTK_LABEL (gui.topic_label), PANGO_ELLIPSIZE_END);
-#else
-	gtk_label_set_line_wrap (GTK_LABEL (gui.topic_label), TRUE);
-#endif
 
 	/* Hook up accelerators for pgup/pgdn */
 	{
@@ -510,15 +562,32 @@ initialize_main_window ()
 		g_closure_unref (closure);
 
 		/* alt-+ */
-		closure = g_cclosure_new (G_CALLBACK (on_discussion_plus_activate), NULL, NULL);
+		closure = g_cclosure_new (G_CALLBACK (on_go_next_discussion_activate), NULL, NULL);
 		gtk_accel_group_connect (discussion_accel, gdk_keyval_from_name ("equal"), GDK_MOD1_MASK, GTK_ACCEL_VISIBLE, closure);
 		g_closure_unref (closure);
 
 		/* alt-- */
-		closure = g_cclosure_new (G_CALLBACK (on_discussion_minus_activate), NULL, NULL);
+		closure = g_cclosure_new (G_CALLBACK (on_go_previous_discussion_activate), NULL, NULL);
 		gtk_accel_group_connect (discussion_accel, gdk_keyval_from_name ("minus"), GDK_MOD1_MASK, GTK_ACCEL_VISIBLE, closure);
 		g_closure_unref (closure);
 
+		/* We've had a couple of calls to hook up Ctrl-Alt-PgUp and
+		 * Ctrl-Alt-PgDown to discussion navigation. As far as I can
+		 * tell this is not HIG compliant, but for the time being we'll
+		 * put it in. It's easy enough to delete it later.
+		 */
+		closure = g_cclosure_new (G_CALLBACK (on_go_next_discussion_activate), NULL, NULL);
+		gtk_accel_group_connect (discussion_accel, GDK_Page_Down,
+				GDK_MOD1_MASK | GDK_CONTROL_MASK , GTK_ACCEL_VISIBLE, closure);
+
+		g_closure_unref (closure);
+
+		closure = g_cclosure_new (G_CALLBACK (on_go_previous_discussion_activate), NULL, NULL);
+		gtk_accel_group_connect (discussion_accel, GDK_Page_Up,
+				GDK_MOD1_MASK | GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE, closure);
+
+		g_closure_unref (closure);
+
 		/* Add the accelgroup to the main window. */
 		gtk_window_add_accel_group (GTK_WINDOW (gui.main_window), discussion_accel);
 	}
@@ -528,8 +597,27 @@ initialize_main_window ()
 	g_signal_connect (G_OBJECT (widget), "activate", G_CALLBACK (find_next), NULL);
 	g_signal_connect (G_OBJECT (widget), "key-press-event", G_CALLBACK (close_find_key), NULL);
 	g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (clear_find), NULL);
+	widget2 = glade_xml_get_widget (gui.xml, "find_next_button");
+	g_signal_connect (G_OBJECT (widget2), "clicked", G_CALLBACK (find_button_next), widget);
 	widget = glade_xml_get_widget (gui.xml, "find close button");
 	g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (close_find_button), NULL);
+
+	/* connect nickname button */
+	widget = glade_xml_get_widget (gui.xml, "nickname");
+	gtk_button_set_use_underline (GTK_BUTTON (widget), FALSE);
+	g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (on_nickname_clicked), NULL);
+
+	/* initialize command completion */
+	/* FIXME - would be nice to find somewhere else for this */
+	command_completion = g_completion_new (NULL);
+	// FIXME - need to convert command_list to a GList
+	//g_completion_add_items (command_completion, command_list);
+	for (i = 0; xc_cmds[i].name != NULL; i++) {
+		tmp = g_list_prepend (tmp, xc_cmds[i].name);
+	}
+	g_list_reverse (tmp);
+	g_completion_add_items (command_completion, tmp);
+	g_list_free (tmp);
 }
 
 void
@@ -564,14 +652,10 @@ run_main_window ()
 	gtk_widget_show (GTK_WIDGET (gui.main_window));
 
 	/* Temporarily disable menu items */
-	widget = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/InsertMenu/InsertColorCodeItem");
-	gtk_widget_set_sensitive (widget, FALSE);
 	widget = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/NetworkMenu/NetworkInformationItem");
 	gtk_widget_set_sensitive (widget, FALSE);
 	widget = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/NetworkMenu/NetworkUsersItem");
 	gtk_widget_set_sensitive (widget, FALSE);
-	widget = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/DiscussionMenu/DiscussionSaveItem");
-	gtk_widget_set_sensitive (widget, FALSE);
 	widget = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/DiscussionMenu/DiscussionBansItem");
 	gtk_widget_set_sensitive (widget, FALSE);
 }
@@ -654,12 +738,6 @@ on_edit_preferences_activate (GtkAction *action, gpointer data)
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
-static void
-on_insert_color_code_activate (GtkAction *action, gpointer data)
-{
-	/* FIXME: implement */
-}
-
 static void
 on_network_information_activate (GtkAction *action, gpointer data)
 {
@@ -683,11 +761,12 @@ on_network_disconnect_activate (GtkAction *action, gpointer data)
 static void
 on_network_close_activate (GtkAction *actoin, gpointer data)
 {
+	GtkTreeIter parent, *iter;
 	session *sess = gui.current_session;
 	if (sess == NULL)
 		return;
 
-	GtkTreeIter parent, *iter = navigation_model_get_unsorted_iter (gui.tree_model, sess);
+	iter = navigation_model_get_unsorted_iter (gui.tree_model, sess);
 
 	sess->server->disconnect (sess, TRUE, -1);
 
@@ -721,7 +800,7 @@ on_network_users_activate (GtkAction *action, gpointer data)
 static void
 on_discussion_save_activate (GtkAction *action, gpointer data)
 {
-	/* FIXME: implement */
+	save_transcript ();
 }
 
 static void
@@ -776,12 +855,6 @@ on_discussion_find_activate (GtkAction *action, gpointer data)
 	gtk_widget_grab_focus (widget);
 }
 
-static void
-on_discussion_find_next_activate (GtkAction *action, gpointer data)
-{
-	/* FIXME: implement */
-}
-
 static void
 on_discussion_clear_window_activate (GtkAction *action, gpointer data)
 {
@@ -810,13 +883,13 @@ on_go_next_network_activate(GtkAction *action, gpointer data)
 static void
 on_go_previous_discussion_activate (GtkAction *action, gpointer data)
 {
-	navigation_tree_select_prev_channel (gui.server_tree, TRUE);
+	navigation_tree_select_prev_channel (gui.server_tree);
 }
 
 static void
 on_go_next_discussion_activate (GtkAction *action, gpointer data)
 {
-	navigation_tree_select_next_channel (gui.server_tree, TRUE);
+	navigation_tree_select_next_channel (gui.server_tree);
 }
 
 static void
@@ -825,18 +898,6 @@ on_discussion_jump_activate (GtkAccelGroup *accelgroup, GObject *arg1, guint arg
 	navigation_tree_select_nth_channel (gui.server_tree, GPOINTER_TO_INT (data));
 }
 
-static void
-on_discussion_plus_activate(GtkAccelGroup *accelgroup, GObject *arg1, guint arg2, GdkModifierType arg3, gpointer data)
-{
-	navigation_tree_select_next_channel (gui.server_tree, FALSE);
-}
-
-static void
-on_discussion_minus_activate (GtkAccelGroup *accelgroup, GObject *arg1, guint arg2, GdkModifierType arg3, gpointer data)
-{
-	navigation_tree_select_prev_channel (gui.server_tree, FALSE);
-}
-
 static void
 on_pgup (GtkAccelGroup *accelgroup, GObject *arg1, guint arg2, GdkModifierType arg3, gpointer data)
 {
@@ -873,6 +934,18 @@ on_pgdn (GtkAccelGroup *accelgroup, GObject *arg1, guint arg2, GdkModifierType a
 	gtk_adjustment_set_value (adj, value);
 }
 
+static void
+on_help_contents_activate (GtkAction *action, gpointer data)
+{
+	GError *error = NULL;
+
+	gnome_help_display_with_doc_id (NULL, NULL, "xchat-gnome.xml", NULL, &error);
+	if (error) {
+		error_dialog ("Error showing help", error->message);
+		g_error_free (error);
+	}
+}
+
 static void
 on_help_about_activate (GtkAction *action, gpointer data)
 {
@@ -889,6 +962,50 @@ on_text_entry_activate (GtkWidget *widget, gpointer data)
 	g_free (entry_text);
 }
 
+static void
+nickname_get_str_response (GtkDialog *dialog, gint arg1, gpointer entry)
+{
+	gchar *text, *buf;
+
+	text = (gchar *) gtk_entry_get_text (GTK_ENTRY (entry));
+
+	if (arg1 == GTK_RESPONSE_ACCEPT || arg1 == GTK_RESPONSE_OK) {
+		buf = g_strdup_printf ("nick %s", text);
+		handle_command (current_sess, buf, FALSE);
+		g_free (buf);
+	}
+	gtk_widget_hide_all (GTK_WIDGET (dialog));
+}
+
+static void
+nickname_str_enter (GtkWidget *entry, GtkWidget *dialog)
+{
+	gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
+}
+
+static void
+on_nickname_clicked (GtkButton *widget, gpointer user_data)
+{
+	GtkWidget *dialog;
+	GtkWidget *entry;
+
+	if (!current_sess)
+		return;
+
+	dialog = glade_xml_get_widget (gui.xml, "nickname dialog");
+	entry = glade_xml_get_widget (gui.xml, "nickname dialog entry");
+
+	gtk_entry_set_text (GTK_ENTRY (entry), current_sess->server->nick);
+	g_signal_connect (G_OBJECT (entry), "activate",
+						 	G_CALLBACK (nickname_str_enter),
+						 	dialog);
+	g_signal_connect (G_OBJECT (dialog), "response",
+						   G_CALLBACK (nickname_get_str_response),
+						   entry);
+
+	gtk_widget_show_all (dialog);
+}
+
 static void
 history_key_down (GtkEntry *entry)
 {
@@ -911,6 +1028,75 @@ history_key_up (GtkEntry *entry)
 	}
 }
 
+static gboolean
+tab_complete_command (GtkEntry *entry)
+{
+	int cursor, length, pos;
+	char *prefix, *new_prefix, *printtext, *npt = NULL;
+	const gchar *text;
+	GList *options, *list;
+	session_gui *tgui;
+
+	cursor = gtk_editable_get_position (GTK_EDITABLE (entry));
+	prefix = g_new0 (char, cursor);
+	text = gtk_entry_get_text (entry);
+	strncpy (prefix, &text[1], cursor - 1);
+	length = strlen (text);
+
+	options = g_completion_complete (command_completion, prefix, &new_prefix);
+
+	if (g_list_length (options) == 0) {
+		/* no matches */
+		g_free (prefix);
+		return TRUE;
+	}
+
+	if (g_list_length (options) == 1) {
+		/* one match */
+
+		if (length - cursor == 0) {
+			/* at the end of the entry, just insert */
+
+			npt = g_strdup_printf ("/%s ", (char *) options->data);
+			pos = strlen (npt);
+		} else {
+			npt = g_strdup_printf ("/%s %s", (char *) options->data, &text[cursor]);
+			pos = strlen ((char *) options->data) + 2;
+		}
+		gtk_entry_set_text (entry, npt);
+		gtk_editable_set_position (GTK_EDITABLE (entry), pos);
+		g_free (npt);
+		g_free (prefix);
+		return TRUE;
+	} else {
+		/* more than one match - print a list of options
+		 * to the window and update the prefix
+		 */
+		list = options;
+		printtext = g_strdup ((char *) list->data);
+		for (list = g_list_next (list); list; list = g_list_next (list)) {
+			npt = g_strdup_printf ("%s %s", printtext, (char *) list->data);
+			g_free (printtext);
+			printtext = npt;
+		}
+		tgui = (session_gui *) gui.current_session->gui;
+		text_gui_print (tgui->buffer, (guchar *) printtext, TRUE);
+		g_free (printtext);
+		if (strcasecmp (prefix, new_prefix) != 0) {
+			/* insert the new prefix into the entry */
+			npt = g_strdup_printf ("/%s%s", new_prefix, &text[cursor]);
+			gtk_entry_set_text (entry, npt);
+			g_free (npt);
+			gtk_editable_set_position (GTK_EDITABLE (entry), strlen (new_prefix));
+		}
+		g_free (prefix);
+		g_free (npt);
+		return TRUE;
+	}
+
+	return TRUE;
+}
+
 static gboolean
 tab_complete_nickname (GtkEntry *entry, int start)
 {
@@ -980,7 +1166,7 @@ tab_complete_nickname (GtkEntry *entry, int start)
 		 */
 		list = options;
 		printtext = g_strdup ((char *) list->data);
-		for (list = list->next; list; list = list->next) {
+		for (list = g_list_next (list); list; list = g_list_next (list)) {
 			npt = g_strdup_printf ("%s %s", printtext, (char *) list->data);
 			g_free (printtext);
 			printtext = npt;
@@ -994,12 +1180,13 @@ tab_complete_nickname (GtkEntry *entry, int start)
 			npt = g_strdup_printf ("%s%s%s", text, new_prefix, &text[cursor]);
 			gtk_entry_set_text (entry, npt);
 			g_free (npt);
-			gtk_editable_set_position (GTK_EDITABLE(entry), start + strlen(new_prefix));
+			gtk_editable_set_position (GTK_EDITABLE (entry), start + strlen (new_prefix));
 		}
-		g_free(text);
+		g_free (text);
 		g_free (prefix);
 		return TRUE;
 	}
+	return TRUE;
 }
 
 static gboolean
@@ -1032,7 +1219,7 @@ tab_complete (GtkEntry *entry)
 
 		/* check if we can match a command */
 		if (start == 0 && text[0] == '/') {
-		    /* TODO: Something? */
+			return tab_complete_command (entry);
 		}
 
 		/* check if we can match a nickname */
@@ -1271,7 +1458,6 @@ entry_context (GtkEntry *entry, GtkMenu *menu, gpointer user_data)
 	gtk_widget_show_all (submenu);
 }
 
-#if (GTK_CHECK_VERSION(2,5,0))
 static void
 on_expand_topic (GtkExpander *expander, gpointer data)
 {
@@ -1283,4 +1469,3 @@ on_expand_topic (GtkExpander *expander, gpointer data)
 		gtk_label_set_line_wrap (GTK_LABEL (gui.topic_label), TRUE);
 	}
 }
-#endif
