@@ -34,10 +34,6 @@ extern GdkPixbuf *pix_newdata;
 extern GdkPixbuf *pix_nicksaid;
 extern GdkPixbuf *pix_msgsaid;
 
-extern GdkPixbuf *pix_prefs_irc;
-extern GdkPixbuf *pix_prefs_colors;
-extern GdkPixbuf *pix_prefs_dcc;
-extern GdkPixbuf *pix_prefs_networks;
 extern GdkPixbuf *pix_prefs_plugins;
 extern GdkPixbuf *pix_prefs_keybindings;
 
