@@ -1,5 +1,5 @@
 /*
- * preferences-page-plugins.h - helpers for the plugins preferences page
+ * preferences-page-plugins.c - helpers for the plugins preferences page
  *
  * Copyright (C) 2004-2005 xchat-gnome team
  *
@@ -114,7 +114,7 @@ load_unload (char *filename, gboolean loaded, PreferencesPluginsPage *page, GtkT
 
 	} else {
 		/* Load the plugin. */
-		gchar *err = load_plugin (gui.current_session, filename, NULL);
+		gchar *err = load_plugin (gui.current_session, filename, NULL, TRUE, FALSE);
 		if (err == NULL) {
 			gtk_list_store_set (page->plugin_store, &iter, 4, TRUE, -1);
 			enabled_plugins = g_slist_append (enabled_plugins, filename);
@@ -254,7 +254,7 @@ preferences_page_plugins_new (gpointer prefs_dialog, GladeXML *xml)
 
 	page->icon = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/plugin-manager.png", NULL);
 	gtk_list_store_append (p->page_store, &iter);
-	gtk_list_store_set (p->page_store, &iter, 0, page->icon, 1, "Scripts and Plugins", 2, 4, -1);
+	gtk_list_store_set (p->page_store, &iter, 0, page->icon, 1, "Scripts and Plugins", 2, 5, -1);
 
 	/*                                          name,          version,       description,   file,          loaded*/
 	page->plugin_store = gtk_list_store_new (5, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
