@@ -192,7 +192,7 @@ find_user (Store *store, struct User *user)
 }
 
 gboolean
-userlist_remove (Userlist *userlist, session *sess, struct User *user)
+userlist_remove_user (Userlist *userlist, session *sess, struct User *user)
 {
 	Store *store = (Store*) g_hash_table_lookup (userlist->stores, sess);
 	GtkTreeIter *iter;
@@ -237,6 +237,8 @@ userlist_update (Userlist *userlist, session *sess, struct User *user)
 	item = g_list_append (NULL, user->nick);
 	g_completion_add_items (store->completion, item);
 	store->completion_items = g_list_concat (store->completion_items, item);
+
+	g_free (nick);
 }
 
 void
@@ -258,7 +260,7 @@ userlist_move (Userlist *userlist, session *sess, struct User *user, int new_row
 }
 
 void
-userlist_clear (Userlist *userlist, session *sess)
+userlist_clear_all (Userlist *userlist, session *sess)
 {
 	Store *store = g_hash_table_lookup (userlist->stores, sess);
 
