@@ -42,6 +42,8 @@
 #include "../common/xchatc.h"
 #include "../common/outbound.h"
 #include "../common/util.h"
+#include "../common/fe.h"
+#include "../common/server.h"
 #include "gtkutil.h"
 #include "maingui.h"
 
@@ -296,7 +298,7 @@ chanlist_do_refresh (struct server *serv)
 
 		handle_command (serv->server_session, "list", FALSE);
 	} else
-		gtkutil_simpledialog ("Not connected.");
+		fe_message (_("Not connected."), FE_MSG_ERROR);
 }
 
 static void
@@ -506,7 +508,7 @@ chanlist_filereq_done (struct server *serv, char *file)
 	if (fh == -1)
 		return;
 
-	snprintf (buf, sizeof buf, "X-Chat Channel List: %s - %s\n",
+	snprintf (buf, sizeof buf, "XChat Channel List: %s - %s\n",
 				 serv->servername, ctime (&t));
 	write (fh, buf, strlen (buf));
 
@@ -533,7 +535,7 @@ chanlist_save (GtkWidget * wid, struct server *serv)
 	if (!gtk_clist_get_text
 		 (GTK_CLIST (serv->gui->chanlist_list), 0, 0, &temp))
 	{
-		gtkutil_simpledialog (_("I can't save an empty list!"));
+		fe_message (_("I can't save an empty list!"), FE_MSG_ERROR);
 		return;
 	}
 	gtkutil_file_req (_("Select an output filename"), chanlist_filereq_done,
@@ -588,7 +590,7 @@ chanlist_closegui (GtkWidget *wid, server *serv)
 }
 
 void
-chanlist_opengui (struct server *serv)
+chanlist_opengui (struct server *serv, int do_refresh)
 {
 	gchar *titles[3];
 	GtkWidget *frame, *vbox, *hbox, *table, *wid;
@@ -604,7 +606,7 @@ chanlist_opengui (struct server *serv)
 		return;
 	}
 
-	snprintf (tbuf, sizeof tbuf, _("X-Chat: Channel List (%s)"),
+	snprintf (tbuf, sizeof tbuf, _("XChat: Channel List (%s)"),
 				 serv->servername);
 
 	serv->gui->chanlist_data_stored_rows = NULL;
@@ -616,7 +618,7 @@ chanlist_opengui (struct server *serv)
 		serv->gui->chanlist_maxusers = 9999;
 
 	serv->gui->chanlist_window =
-		mg_create_generic_tab ("chanlist", tbuf, FALSE, TRUE, chanlist_closegui,
+		mg_create_generic_tab ("ChanList", tbuf, FALSE, TRUE, chanlist_closegui,
 								serv, 450, 356, &vbox, serv);
 
 	frame = gtk_frame_new (_("List display options:"));
@@ -640,7 +642,7 @@ chanlist_opengui (struct server *serv)
 	wid = gtk_spin_button_new_with_range (0, 999999, 1);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (wid),
 										serv->gui->chanlist_minusers);
-	g_signal_connect (G_OBJECT (wid), "value-changed",
+	g_signal_connect (G_OBJECT (wid), "value_changed",
 							G_CALLBACK (chanlist_minusers), serv);
 	gtk_table_attach (GTK_TABLE (table), wid, 1, 2, 0, 1,
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
@@ -655,7 +657,7 @@ chanlist_opengui (struct server *serv)
 	wid = gtk_spin_button_new_with_range (0, 999999, 1);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (wid),
 										serv->gui->chanlist_maxusers);
-	g_signal_connect (G_OBJECT (wid), "value-changed",
+	g_signal_connect (G_OBJECT (wid), "value_changed",
 							G_CALLBACK (chanlist_maxusers), serv);
 	gtk_table_attach (GTK_TABLE (table), wid, 1, 2, 2, 3,
 							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
@@ -765,4 +767,7 @@ chanlist_opengui (struct server *serv)
 	chanlist_reset_sort_vars (serv);
 
 	gtk_widget_show (serv->gui->chanlist_window);
+
+	if (do_refresh)
+		chanlist_do_refresh (serv);
 }
