@@ -34,6 +34,7 @@
 #include <gtk/gtkentry.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkimagemenuitem.h>
+#include <gtk/gtkradiomenuitem.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenubar.h>
@@ -73,12 +74,12 @@ static GSList *submenu_list;
 
 enum
 {
-	M_MENU,
+	M_MENUITEM,
 	M_NEWMENU,
 	M_END,
 	M_SEP,
 	M_MENUTOG,
-	M_MENUD,
+	M_MENURADIO,
 	M_MENUSTOCK,
 	M_MENUPIX,
 	M_MENUSUB
@@ -89,10 +90,11 @@ struct mymenu
 	char *text;
 	void *callback;
 	char *image;
-	short type;
-	unsigned int state:1;
-	unsigned int activate:1;
-	guint key;
+	unsigned char type;	/* M_XXX */
+	unsigned char id;		/* MENU_ID_XXX (menu.h) */
+	unsigned char state;	/* ticked or not? */
+	unsigned char sensitive;	/* shaded out? */
+	guint key;				/* GDK_x */
 };
 
 
@@ -125,7 +127,7 @@ nick_command_parse (session *sess, char *cmd, char *nick, char *allnick)
 			host = buf + 1;
 	} else*/
 	{
-		user = find_name (sess, nick);
+		user = userlist_find (sess, nick);
 		if (user && user->hostname)
 			host = strchr (user->hostname, '@') + 1;
 	}
@@ -161,7 +163,7 @@ userlist_button_cb (GtkWidget * button, char *cmd)
 	{
 		/* fake a selection */
 		nicks = malloc (sizeof (char *) * 2);
-		nicks[0] = sess->channel;
+		nicks[0] = g_strdup (sess->channel);
 		nicks[1] = NULL;
 		num_sel = 1;
 	} else
@@ -203,9 +205,13 @@ userlist_button_cb (GtkWidget * button, char *cmd)
 		nick_command_parse (sess, cmd, nick, allnicks);
 	}
 
-	if (nicks)
-		free (nicks);
+	while (num_sel)
+	{
+		num_sel--;
+		g_free (nicks[num_sel]);
+	}
 
+	free (nicks);
 	free (allnicks);
 }
 
@@ -439,9 +445,10 @@ menu_create (GtkWidget *menu, GSList *list, char *target, int check_path)
 }
 
 static void
-menu_destroy (GtkObject *object, gpointer objtounref)
+menu_destroy (GtkWidget *menu, gpointer objtounref)
 {
-	gtk_widget_destroy (GTK_WIDGET (object));
+	gtk_widget_destroy (menu);
+	g_object_unref (menu);
 	if (objtounref)
 		g_object_unref (G_OBJECT (objtounref));
 }
@@ -454,13 +461,12 @@ menu_popup (GtkWidget *menu, GdkEventButton *event, gpointer objtounref)
 		gtk_menu_set_screen (GTK_MENU (menu), gdk_drawable_get_screen (event->window));
 #endif
 
+	g_object_ref (menu);
+	gtk_object_sink (GTK_OBJECT (menu));
 	g_signal_connect (G_OBJECT (menu), "selection-done",
 							G_CALLBACK (menu_destroy), objtounref);
-	if (event == NULL)
-		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 0, 0);
-	else
-		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
-							 0, event->time);
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
+						 0, event ? event->time : 0);
 }
 
 static char *str_copy = 0;		/* for all pop-up menus */
@@ -488,7 +494,9 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 		menu_quick_item (0, 0, menu, 1, 0);
 	} else
 	{
-		user = find_name_global (current_sess->server, nick);
+		user = userlist_find (sess, nick);	/* lasttalk is channel specific */
+		if (!user)
+			user = userlist_find_global (current_sess->server, nick);
 		if (user)
 		{
 			submenu = menu_quick_sub (nick, menu, NULL, 1, -1);
@@ -496,6 +504,10 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 			/* let the translators tweak this if need be */
 			fmt = _("<tt><b>%-11s</b></tt> %s");
 
+			snprintf (buf, sizeof (buf), fmt, _("Real Name:"),
+						user->realname ? user->realname : _("Unknown"));
+			menu_quick_item (0, buf, submenu, 2, 0);
+
 			snprintf (buf, sizeof (buf), fmt, _("User:"),
 						user->hostname ? user->hostname : _("Unknown"));
 			menu_quick_item (0, buf, submenu, 2, 0);
@@ -504,29 +516,34 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 						user->hostname ? country(user->hostname) : _("Unknown"));
 			menu_quick_item (0, buf, submenu, 2, 0);
 
-			snprintf (buf, sizeof (buf), fmt, _("Real Name:"),
-						user->realname ? user->realname : _("Unknown"));
-			menu_quick_item (0, buf, submenu, 2, 0);
-
 			snprintf (buf, sizeof (buf), fmt, _("Server:"),
 						user->servername ? user->servername : _("Unknown"));
 			menu_quick_item (0, buf, submenu, 2, 0);
 
 			if (user->away)
 			{
-				away = find_away_message (current_sess->server, nick);
+				away = server_away_find_message (current_sess->server, nick);
 				if (away)
 				{
-					snprintf (buf, sizeof (buf), fmt, _("Away Msg:"),
-								 away->message ? away->message : _("Unknown"));
+					char *msg = strip_color (away->message ? away->message : _("Unknown"), -1, 1, 1);
+					snprintf (buf, sizeof (buf), fmt, _("Away Msg:"), msg);
 					menu_quick_item (0, buf, submenu, 2, 0);
+					free (msg);
 				}
 			}
 
-			snprintf (buf, sizeof (buf), fmt, _("Last Msg:"),
-						user->lasttalk ? ctime (&(user->lasttalk)) : _("Unknown"));
 			if (user->lasttalk)
-				buf[strlen (buf) - 1] = 0;
+			{
+				char min[96];
+
+				snprintf (min, sizeof (min), _("%u minutes ago"),
+							(unsigned int) ((time (0) - user->lasttalk) / 60));
+				snprintf (buf, sizeof (buf), fmt, _("Last Msg:"), min);
+			} else
+			{
+				snprintf (buf, sizeof (buf), fmt, _("Last Msg:"), _("Unknown"));
+			}
+
 			menu_quick_item (0, buf, submenu, 2, 0);
 
 			menu_quick_endsub ();
@@ -541,23 +558,71 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick, int num_sel)
 	menu_popup (menu, event, NULL);
 }
 
+/* stuff for the View menu */
+
 static void
-menu_showhide (void)
+menu_showhide_cb (session *sess)
+{
+	if (prefs.hidemenu)
+		gtk_widget_hide (sess->gui->menu);
+	else
+		gtk_widget_show (sess->gui->menu);
+}
+
+static void
+menu_topic_showhide_cb (session *sess)
+{
+	if (prefs.topicbar)
+		gtk_widget_show (sess->gui->topic_bar);
+	else
+		gtk_widget_hide (sess->gui->topic_bar);
+}
+
+static void
+menu_ulbuttons_showhide_cb (session *sess)
+{
+	if (prefs.userlistbuttons)
+		gtk_widget_show (sess->gui->button_box);
+	else
+		gtk_widget_hide (sess->gui->button_box);
+}
+
+static void
+menu_cmbuttons_showhide_cb (session *sess)
+{
+	switch (sess->type)
+	{
+	case SESS_CHANNEL:
+		if (prefs.chanmodebuttons)
+			gtk_widget_show (sess->gui->topicbutton_box);
+		else
+			gtk_widget_hide (sess->gui->topicbutton_box);
+		break;
+	default:
+		gtk_widget_hide (sess->gui->topicbutton_box);
+	}
+}
+
+static void
+menu_setting_foreach (void (*callback) (session *), int id, guint state)
 {
 	session *sess;
 	GSList *list;
+	int maindone = FALSE;	/* do it only once for EVERY tab */
 
 	list = sess_list;
 	while (list)
 	{
 		sess = list->data;
 
-		if (sess->gui->menu)
+		if (!sess->gui->is_tab || !maindone)
 		{
-			if (prefs.hidemenu)
-				gtk_widget_hide (sess->gui->menu);
-			else
-				gtk_widget_show (sess->gui->menu);
+			if (sess->gui->is_tab)
+				maindone = TRUE;
+			if (id != -1)
+				GTK_CHECK_MENU_ITEM (sess->gui->menu_item[id])->active = state;
+			if (callback)
+				callback (sess);
 		}
 
 		list = list->next;
@@ -568,65 +633,51 @@ void
 menu_bar_toggle (void)
 {
 	prefs.hidemenu = !prefs.hidemenu;
-	menu_showhide ();
+	menu_setting_foreach (menu_showhide_cb, MENU_ID_MENUBAR, !prefs.hidemenu);
 }
 
 static void
-menu_middle_cb (GtkWidget *item, gpointer userdata)
+menu_bar_toggle_cb (void)
 {
-	switch (GPOINTER_TO_INT (userdata))
-	{
-	case 0:
-		menu_bar_toggle ();
-		break;
-	case 1:
-		mg_topic_showhide (current_sess);
-		break;
-	case 2:
-		prefs.hideuserlist = !prefs.hideuserlist;
-		mg_userlist_showhide (current_sess, !prefs.hideuserlist);
-		break;
-	case 3:
-		prefs.chanmodebuttons = !prefs.chanmodebuttons;
-		mg_chanmodebuttons_showhide (current_sess, prefs.chanmodebuttons);
-		break;
-	case 4:
-		prefs.userlistbuttons = !prefs.userlistbuttons;
-		if (prefs.userlistbuttons)
-			gtk_widget_show (current_sess->gui->button_box);
-		else
-			gtk_widget_hide (current_sess->gui->button_box);
-	}
+	menu_bar_toggle ();
+	if (prefs.hidemenu)
+		fe_message (_("The Menubar is now hidden. You can show it again"
+						  " by pressing F9 or right-clicking in a blank part of"
+						  " the main text area."), FE_MSG_INFO);
+}
+
+static void
+menu_topicbar_toggle (GtkWidget *wid, gpointer ud)
+{
+	prefs.topicbar = !prefs.topicbar;
+	menu_setting_foreach (menu_topic_showhide_cb, MENU_ID_TOPICBAR,
+								 prefs.topicbar);
+}
+
+static void
+menu_ulbuttons_toggle (GtkWidget *wid, gpointer ud)
+{
+	prefs.userlistbuttons = !prefs.userlistbuttons;
+	menu_setting_foreach (menu_ulbuttons_showhide_cb, MENU_ID_ULBUTTONS,
+								 prefs.userlistbuttons);
+}
+
+static void
+menu_cmbuttons_toggle (GtkWidget *wid, gpointer ud)
+{
+	prefs.chanmodebuttons = !prefs.chanmodebuttons;
+	menu_setting_foreach (menu_cmbuttons_showhide_cb, MENU_ID_MODEBUTTONS,
+								 prefs.chanmodebuttons);
 }
 
 void
 menu_middlemenu (session *sess, GdkEventButton *event)
 {
-	GtkWidget *menu, *away, *user, *item;
+	GtkWidget *menu;
 	GtkAccelGroup *accel_group;
 
 	accel_group = gtk_accel_group_new ();
-	menu = menu_create_main (accel_group, FALSE, sess->server->is_away, !sess->gui->is_tab, &away, &user);
-
-	menu_quick_item (0, 0, menu, 1, 0);	/* sep */
-
-	menu_toggle_item (_("Menu Bar"), menu, menu_middle_cb, 0, !prefs.hidemenu);
-	menu_toggle_item (_("Topic Bar"), menu, menu_middle_cb,
-							GINT_TO_POINTER (1), prefs.topicbar);
-
-	if (!prefs.paned_userlist)
-		menu_toggle_item (_("User List"), menu, menu_middle_cb,
-								GINT_TO_POINTER (2), !prefs.hideuserlist);
-
-	item = menu_toggle_item (_("Mode Buttons"), menu, menu_middle_cb,
-									GINT_TO_POINTER (3),
-									prefs.topicbar ? prefs.chanmodebuttons : 0);
-	if (!prefs.topicbar)
-		gtk_widget_set_sensitive (item, FALSE);
-
-	menu_toggle_item (_("User List Buttons"), menu, menu_middle_cb,
-							GINT_TO_POINTER (4), prefs.userlistbuttons);
-
+	menu = menu_create_main (accel_group, FALSE, sess->server->is_away, !sess->gui->is_tab, NULL);
 	menu_popup (menu, event, accel_group);
 }
 
@@ -670,12 +721,17 @@ menu_urlmenu (GdkEventButton *event, char *url)
 	}
 	menu_quick_item (0, 0, menu, 1, 0);
 
-	menu_quick_item_with_callback (copy_to_clipboard_cb, _("Copy selected URL"), menu, str_copy);
+	menu_quick_item_with_callback (copy_to_clipboard_cb, _("Copy Selected Link"), menu, str_copy);
+#ifndef WIN32
+	if (!strchr (str_copy, '`'))
+#endif
+	{
 #ifdef WIN32
-	menu_quick_item_with_callback (open_url_cb, "Open URL", menu, str_copy);
+		menu_quick_item_with_callback (open_url_cb, _("Open Link in Browser"), menu, str_copy);
 #endif
 
-	menu_create (menu, urlhandler_list, str_copy, TRUE);
+		menu_create (menu, urlhandler_list, str_copy, TRUE);
+	}
 	menu_popup (menu, event, NULL);
 }
 
@@ -758,8 +814,8 @@ menu_settings (GtkWidget * wid, gpointer none)
 static void
 menu_usermenu (void)
 {
-	editlist_gui_open (usermenu_list, _("X-Chat: User menu"), "usermenu",
-							 "usermenu.conf", 0);
+	editlist_gui_open (NULL, NULL, usermenu_list, _("XChat: User menu"),
+							 "usermenu", "usermenu.conf", 0);
 }
 
 static void
@@ -790,25 +846,24 @@ usermenu_update (void)
 	int done_main = FALSE;
 	GSList *list = sess_list;
 	session *sess;
+	GtkWidget *menu;
 
 	while (list)
 	{
 		sess = list->data;
+		menu = sess->gui->menu_item[MENU_ID_USERMENU];
 		if (sess->gui->is_tab)
 		{
-			if (!done_main)
+			if (!done_main && menu)
 			{
-				if (sess->gui->user_menu)
-				{
-					usermenu_destroy (sess->gui->user_menu);
-					usermenu_create (sess->gui->user_menu);
-					done_main = TRUE;
-				}
+				usermenu_destroy (menu);
+				usermenu_create (menu);
+				done_main = TRUE;
 			}
-		} else
+		} else if (menu)
 		{
-			usermenu_destroy (sess->gui->user_menu);
-			usermenu_create (sess->gui->user_menu);
+			usermenu_destroy (menu);
+			usermenu_create (menu);
 		}
 		list = list->next;
 	}
@@ -874,10 +929,10 @@ menu_autodccsend (GtkWidget * wid, gpointer none)
 	{
 		if (!strcmp ((char *)g_get_home_dir (), prefs.dccdir))
 		{
-			gtkutil_simpledialog (_("*WARNING*\n"
-										 "Auto accepting DCC to your home directory\n"
-										 "can be dangerous and is exploitable. Eg:\n"
-										 "Someone could send you a .bash_profile"));
+			fe_message (_("*WARNING*\n"
+							 "Auto accepting DCC to your home directory\n"
+							 "can be dangerous and is exploitable. Eg:\n"
+							 "Someone could send you a .bash_profile"), FE_MSG_WARN);
 		}
 	}
 #endif
@@ -892,7 +947,7 @@ menu_detach (GtkWidget * wid, gpointer none)
 static void
 menu_close (GtkWidget * wid, gpointer none)
 {
-	mg_x_click_cb (NULL, NULL);
+	mg_close_sess (current_sess);
 }
 
 static void
@@ -972,6 +1027,21 @@ menu_servernotice (GtkWidget * wid, gpointer none)
 	}
 }
 
+static void
+menu_disconnect (GtkWidget * wid, gpointer none)
+{
+	handle_command (current_sess, "DISCON", FALSE);
+}
+
+static void
+menu_reconnect (GtkWidget * wid, gpointer none)
+{
+	if (current_sess->server->hostname[0])
+		handle_command (current_sess, "RECONNECT", FALSE);
+	else
+		fe_serverlist_open (current_sess);
+}
+
 static void
 menu_away (GtkWidget * wid, gpointer none)
 {
@@ -1002,7 +1072,7 @@ menu_savedefault (GtkWidget * wid, gpointer none)
 {
 	palette_save ();
 	if (save_config ())
-		gtkutil_simpledialog (_("Settings saved."));
+		fe_message (_("Settings saved."), FE_MSG_INFO);
 }
 #endif
 
@@ -1086,33 +1156,33 @@ menu_pluginlist (void)
                            "%s  =  the URL string\n\n"\
                            "Putting a ! infront of the command\n"\
                            "indicates it should be sent to a\n"\
-                           "shell instead of X-Chat")
+                           "shell instead of XChat")
 
 static void
 menu_usercommands (void)
 {
-	editlist_gui_open (command_list, _("X-Chat: User Defined Commands"),
+	editlist_gui_open (NULL, NULL, command_list, _("XChat: User Defined Commands"),
 							 "commands", "commands.conf", usercommands_help);
 }
 
 static void
 menu_ulpopup (void)
 {
-	editlist_gui_open (popup_list, _("X-Chat: Userlist Popup menu"), "popup",
+	editlist_gui_open (NULL, NULL, popup_list, _("XChat: Userlist Popup menu"), "popup",
 							 "popup.conf", ulbutton_help);
 }
 
 static void
 menu_rpopup (void)
 {
-	editlist_gui_open (replace_list, _("X-Chat: Replace"), "replace",
+	editlist_gui_open (_("Text"), _("Replace with"), replace_list, _("XChat: Replace"), "replace",
 							 "replace.conf", 0);
 }
 
 static void
 menu_urlhandlers (void)
 {
-	editlist_gui_open (urlhandler_list, _("X-Chat: URL Handlers"), "urlhandlers",
+	editlist_gui_open (NULL, NULL, urlhandler_list, _("XChat: URL Handlers"), "urlhandlers",
 							 "urlhandlers.conf", url_help);
 }
 
@@ -1131,21 +1201,21 @@ menu_keypopup (void)
 static void
 menu_ulbuttons (void)
 {
-	editlist_gui_open (button_list, _("X-Chat: Userlist buttons"), "buttons",
+	editlist_gui_open (NULL, NULL, button_list, _("XChat: Userlist buttons"), "buttons",
 							 "buttons.conf", ulbutton_help);
 }
 
 static void
 menu_dlgbuttons (void)
 {
-	editlist_gui_open (dlgbutton_list, _("X-Chat: Dialog buttons"), "dlgbuttons",
+	editlist_gui_open (NULL, NULL, dlgbutton_list, _("XChat: Dialog buttons"), "dlgbuttons",
 							 "dlgbuttons.conf", dlgbutton_help);
 }
 
 static void
 menu_ctcpguiopen (void)
 {
-	editlist_gui_open (ctcp_list, _("X-Chat: CTCP Replies"), "ctcpreply",
+	editlist_gui_open (NULL, NULL, ctcp_list, _("XChat: CTCP Replies"), "ctcpreply",
 							 "ctcpreply.conf", ctcp_help);
 }
 
@@ -1156,7 +1226,7 @@ menu_reload (void)
 	char *buf = malloc (strlen (default_file ()) + 12);
 	load_config ();
 	sprintf (buf, "%s reloaded.", default_file ());
-	fe_message (buf, FALSE);
+	fe_message (buf, FE_MSG_INFO);
 	free (buf);
 }
 #endif
@@ -1215,14 +1285,9 @@ menu_webpage (GtkWidget *wid, gpointer none)
 }*/
 
 static void
-menu_dcc_recv_win (GtkWidget *wid, gpointer none)
+menu_dcc_win (GtkWidget *wid, gpointer none)
 {
 	fe_dcc_open_recv_win (FALSE);
-}
-
-static void
-menu_dcc_send_win (GtkWidget *wid, gpointer none)
-{
 	fe_dcc_open_send_win (FALSE);
 }
 
@@ -1232,104 +1297,166 @@ menu_dcc_chat_win (GtkWidget *wid, gpointer none)
 	fe_dcc_open_chat_win (FALSE);
 }
 
-static struct mymenu mymenu[] = {
-	{N_("_X-Chat"), 0, 0, M_NEWMENU, 0, 1},
-	{N_("Server List..."), menu_open_server_list, (char *)&pix_book, M_MENUPIX, 0, 1, GDK_s},
-	{0, 0, 0, M_SEP, 0, 0},
+static void
+menu_layout_cb (GtkWidget *item, gpointer none)
+{
+	if (GTK_CHECK_MENU_ITEM (item)->active)
+	{
+		prefs.tab_layout = 0;
+		menu_setting_foreach (NULL, MENU_ID_LAYOUT_TABS, 1);
+		menu_setting_foreach (NULL, MENU_ID_LAYOUT_TREE, 0);
+		mg_change_layout (0);
+	} else
+	{
+		prefs.tab_layout = 2;
+		menu_setting_foreach (NULL, MENU_ID_LAYOUT_TABS, 0);
+		menu_setting_foreach (NULL, MENU_ID_LAYOUT_TREE, 1);
+		mg_change_layout (2);
+	}
+}
 
-	{N_("New"), 0, GTK_STOCK_NEW, M_MENUSUB, 0, 1},
-	{N_("Server Tab..."), menu_newserver_tab, 0, M_MENU, 0, 1, GDK_t},
-	{N_("Channel Tab..."), menu_newchannel_tab, 0, M_MENU, 0, 1},
-	{N_("Server Window..."), menu_newserver_window, 0, M_MENU, 0, 1},
-	{N_("Channel Window..."), menu_newchannel_window, 0, M_MENU, 0, 1},
-	{0, 0, 0, M_END, 0, 0},
-	{0, 0, 0, M_SEP, 0, 0},
+static void
+menu_apply_metres_cb (session *sess)
+{
+	mg_update_meters (sess->gui);
+}
+
+static void
+menu_metres_off (GtkWidget *item, gpointer none)
+{
+	if (GTK_CHECK_MENU_ITEM (item)->active)
+	{
+		prefs.lagometer = 0;
+		prefs.throttlemeter = 0;
+		menu_setting_foreach (menu_apply_metres_cb, -1, 0);
+	}
+}
+
+static void
+menu_metres_text (GtkWidget *item, gpointer none)
+{
+	if (GTK_CHECK_MENU_ITEM (item)->active)
+	{
+		prefs.lagometer = 2;
+		prefs.throttlemeter = 2;
+		menu_setting_foreach (menu_apply_metres_cb, -1, 0);
+	}
+}
+
+static void
+menu_metres_graph (GtkWidget *item, gpointer none)
+{
+	if (GTK_CHECK_MENU_ITEM (item)->active)
+	{
+		prefs.lagometer = 1;
+		prefs.throttlemeter = 1;
+		menu_setting_foreach (menu_apply_metres_cb, -1, 0);
+	}
+}
+
+static void
+menu_metres_both (GtkWidget *item, gpointer none)
+{
+	if (GTK_CHECK_MENU_ITEM (item)->active)
+	{
+		prefs.lagometer = 3;
+		prefs.throttlemeter = 3;
+		menu_setting_foreach (menu_apply_metres_cb, -1, 0);
+	}
+}
+
+static struct mymenu mymenu[] = {
+	{N_("_XChat"), 0, 0, M_NEWMENU, 0, 0, 1},
+	{N_("_Server List..."), menu_open_server_list, (char *)&pix_book, M_MENUPIX, 0, 0, 1, GDK_s},
+	{0, 0, 0, M_SEP, 0, 0, 0},
+
+	{N_("_New"), 0, GTK_STOCK_NEW, M_MENUSUB, 0, 0, 1},
+		{N_("Server Tab..."), menu_newserver_tab, 0, M_MENUITEM, 0, 0, 1, GDK_t},
+		{N_("Channel Tab..."), menu_newchannel_tab, 0, M_MENUITEM, 0, 0, 1},
+		{N_("Server Window..."), menu_newserver_window, 0, M_MENUITEM, 0, 0, 1},
+		{N_("Channel Window..."), menu_newchannel_window, 0, M_MENUITEM, 0, 0, 1},
+		{0, 0, 0, M_END, 0, 0, 0},
+	{0, 0, 0, M_SEP, 0, 0, 0},
 
 #ifdef USE_PLUGIN
-	{N_("Load Plugin or Script..."), menu_loadplugin, GTK_STOCK_REVERT_TO_SAVED, M_MENUSTOCK, 0, 1},
+	{N_("_Load Plugin or Script..."), menu_loadplugin, GTK_STOCK_REVERT_TO_SAVED, M_MENUSTOCK, 0, 0, 1},
 #else
-	{N_("Load Plugin or Script..."), 0, GTK_STOCK_REVERT_TO_SAVED, M_MENUSTOCK, 0, 0},
+	{N_("_Load Plugin or Script..."), 0, GTK_STOCK_REVERT_TO_SAVED, M_MENUSTOCK, 0, 0, 0},
 #endif
-	{0, 0, 0, M_SEP, 0, 0},	/* 11 */
-#ifdef USE_ZVT
-	{N_("New Shell Tab..."), menu_newshell_tab, 0, M_MENU, 0, 1},
-	{0, 0, 0, M_SEP, 0, 0},
-#define menuoffset 0
-#else
-#define menuoffset 2
-#endif
-	{0, menu_detach, GTK_STOCK_REDO, M_MENUSTOCK, 0, 1, GDK_I},	/* 14 */
-	{N_("Close Tab"), menu_close, GTK_STOCK_CLOSE, M_MENUSTOCK, 0, 1, GDK_w},
-	{0, 0, 0, M_SEP, 0, 0},
-	{N_("Quit"), mg_safe_quit, GTK_STOCK_QUIT, M_MENUSTOCK, 0, 1, GDK_q},	/* 17 */
+	{0, 0, 0, M_SEP, 0, 0, 0},	/* 11 */
+#define DETACH_OFFSET (12)
+	{0, menu_detach, GTK_STOCK_REDO, M_MENUSTOCK, 0, 0, 1, GDK_I},	/* 12 */
+#define CLOSE_OFFSET (13)
+	{0, menu_close, GTK_STOCK_CLOSE, M_MENUSTOCK, 0, 0, 1, GDK_w},
+	{0, 0, 0, M_SEP, 0, 0, 0},
+	{N_("_Quit"), mg_safe_quit, GTK_STOCK_QUIT, M_MENUSTOCK, 0, 0, 1, GDK_q},	/* 15 */
 
-	{N_("_IRC"), 0, 0, M_NEWMENU, 0, 1},
-	{N_("Invisible"), menu_invisible, 0, M_MENUTOG, 1, 1},
-	{N_("Receive Wallops"), menu_wallops, 0, M_MENUTOG, 1, 1},
-	{N_("Receive Server Notices"), menu_servernotice, 0, M_MENUTOG, 1, 1},
-	{0, 0, 0, M_SEP, 0, 0},
-	{N_("Marked Away"), menu_away, 0, M_MENUTOG, 0, 1, GDK_a},
-	{0, 0, 0, M_SEP, 0, 0},														/* 24 */
-	{N_("Auto Rejoin when Kicked"), menu_autorejoin, 0, M_MENUTOG, 0, 1},
-	{N_("Auto Reconnect to Server"), menu_autoreconnect, 0, M_MENUTOG, 0, 1},
-	{N_("Never-give-up ReConnect"), menu_autoreconnectonfail, 0, M_MENUTOG, 0, 1},
-	{0, 0, 0, M_SEP, 0, 0},														/* 28 */
-	{N_("Auto Open Dialog Windows"), menu_autodialog, 0, M_MENUTOG, 0, 1},
-	{N_("Auto Accept Direct Chat"), menu_autodccchat, 0, M_MENUTOG, 0, 1},
-	{N_("Auto Accept Files"), menu_autodccsend, 0, M_MENUTOG, 0, 1},
+	{N_("_View"), 0, 0, M_NEWMENU, 0, 0, 1},
+#define MENUBAR_OFFSET (17)
+	{N_("_Menubar"), menu_bar_toggle_cb, 0, M_MENUTOG, MENU_ID_MENUBAR, 0, 1, GDK_F9},
+	{N_("_Topicbar"), menu_topicbar_toggle, 0, M_MENUTOG, MENU_ID_TOPICBAR, 0, 1},
+	{N_("_Userlist Buttons"), menu_ulbuttons_toggle, 0, M_MENUTOG, MENU_ID_ULBUTTONS, 0, 1},
+	{N_("M_ode Buttons"), menu_cmbuttons_toggle, 0, M_MENUTOG, MENU_ID_MODEBUTTONS, 0, 1},
+	{0, 0, 0, M_SEP, 0, 0, 0},
+	{N_("_Layout"), 0, 0, M_MENUSUB, 0, 0, 1},	/* 22 */
+#define TABS_OFFSET (23)
+		{N_("_Tabs"), menu_layout_cb, 0, M_MENURADIO, MENU_ID_LAYOUT_TABS, 0, 1},
+		{N_("T_ree"), 0, 0, M_MENURADIO, MENU_ID_LAYOUT_TREE, 0, 1},
+		{0, 0, 0, M_END, 0, 0, 0},
+	{N_("_Network Meters"), 0, 0, M_MENUSUB, 0, 0, 1},	/* 26 */
+#define METRE_OFFSET (27)
+		{N_("Off"), menu_metres_off, 0, M_MENURADIO, 0, 0, 1},
+		{N_("Graph"), menu_metres_graph, 0, M_MENURADIO, 0, 0, 1},
+		{N_("Text"), menu_metres_text, 0, M_MENURADIO, 0, 0, 1},
+		{N_("Both"), menu_metres_both, 0, M_MENURADIO, 0, 0, 1},
+		{0, 0, 0, M_END, 0, 0, 0},	/* 31 */
 
-	{N_("_Server"), (void *) -1, 0, M_NEWMENU, 0, 1},	/* 32 */
+	{N_("_Server"), 0, 0, M_NEWMENU, 0, 0, 1},
+	{N_("_Disconnect"), menu_disconnect, GTK_STOCK_DISCONNECT, M_MENUSTOCK, MENU_ID_DISCONNECT, 0, 1},
+	{N_("_Reconnect"), menu_reconnect, GTK_STOCK_CONNECT, M_MENUSTOCK, MENU_ID_RECONNECT, 0, 1},
+	{0, 0, 0, M_SEP, 0, 0, 0},
+#define AWAY_OFFSET (36)
+	{N_("Marked Away"), menu_away, 0, M_MENUTOG, MENU_ID_AWAY, 0, 1, GDK_a},
 
-	{N_("S_ettings"), 0, 0, M_NEWMENU, 0, 1},	/* 33 */
-	{N_("Preferences..."), menu_settings, GTK_STOCK_PREFERENCES, M_MENUSTOCK, 0, 1},
+	{N_("_Usermenu"), 0, 0, M_NEWMENU, MENU_ID_USERMENU, 0, 1},	/* 37 */
 
-	{N_("Advanced"), 0, GTK_STOCK_JUSTIFY_LEFT, M_MENUSUB, 0, 1},
-	{N_("Auto Replace..."), menu_rpopup, 0, M_MENU, 0, 1},
-	{N_("CTCP Replies..."), menu_ctcpguiopen, 0, M_MENU, 0, 1},
-	{N_("Dialog Buttons..."), menu_dlgbuttons, 0, M_MENU, 0, 1},
-	{N_("Keyboard Shortcuts..."), menu_keypopup, 0, M_MENU, 0, 1},
-	{N_("Text Events..."), menu_evtpopup, 0, M_MENU, 0, 1},
-	{N_("URL Handlers..."), menu_urlhandlers, 0, M_MENU, 0, 1},
-	{N_("User Commands..."), menu_usercommands, 0, M_MENU, 0, 1},
-	{N_("Userlist Buttons..."), menu_ulbuttons, 0, M_MENU, 0, 1},
-	{N_("Userlist Popup..."), menu_ulpopup, 0, M_MENU, 0, 1},
-	{0, 0, 0, M_END, 0, 0},		/* 45 */
+	{N_("S_ettings"), 0, 0, M_NEWMENU, 0, 0, 1},
+	{N_("Preferences..."), menu_settings, GTK_STOCK_PREFERENCES, M_MENUSTOCK, 0, 0, 1},
 
-#if 0
-	{0, 0, 0, M_SEP, 0, 0},	/* 56 */
-	{N_("Reload Settings"), menu_reload, GTK_STOCK_REVERT_TO_SAVED, M_MENUSTOCK, 0, 1},
-	{0, 0, 0, M_SEP, 0, 0},
-	{N_("Save Settings now"), menu_savedefault, GTK_STOCK_SAVE, M_MENUSTOCK, 0, 1},
-	{N_("Save Settings on exit"), menu_saveexit, 0, M_MENUTOG, 1, 1},
-#endif
+	{N_("Advanced"), 0, GTK_STOCK_JUSTIFY_LEFT, M_MENUSUB, 0, 0, 1},
+		{N_("Auto Replace..."), menu_rpopup, 0, M_MENUITEM, 0, 0, 1},
+		{N_("CTCP Replies..."), menu_ctcpguiopen, 0, M_MENUITEM, 0, 0, 1},
+		{N_("Dialog Buttons..."), menu_dlgbuttons, 0, M_MENUITEM, 0, 0, 1},
+		{N_("Keyboard Shortcuts..."), menu_keypopup, 0, M_MENUITEM, 0, 0, 1},
+		{N_("Text Events..."), menu_evtpopup, 0, M_MENUITEM, 0, 0, 1},
+		{N_("URL Handlers..."), menu_urlhandlers, 0, M_MENUITEM, 0, 0, 1},
+		{N_("User Commands..."), menu_usercommands, 0, M_MENUITEM, 0, 0, 1},
+		{N_("Userlist Buttons..."), menu_ulbuttons, 0, M_MENUITEM, 0, 0, 1},
+		{N_("Userlist Popup..."), menu_ulpopup, 0, M_MENUITEM, 0, 0, 1},
+		{0, 0, 0, M_END, 0, 0, 0},		/* 50 */
 
-	{N_("_Window"), 0, 0, M_NEWMENU, 0, 1},
-	{N_("Ban List..."), menu_banlist, 0, M_MENU, 0, 1},
-	{N_("Channel List..."), menu_chanlist, 0, M_MENU, 0, 1},
-	{N_("Character Chart..."), ascii_open, 0, M_MENU, 0, 1},
-	{N_("Direct Chat..."), menu_dcc_chat_win, 0, M_MENU, 0, 1},
-	{N_("File Receive..."), menu_dcc_recv_win, 0, M_MENU, 0, 1},
-	{N_("File Send..."), menu_dcc_send_win, 0, M_MENU, 0, 1},
-	{N_("Ignore List..."), ignore_gui_open, 0, M_MENU, 0, 1},
-	{N_("Notify List..."), notify_opengui, 0, M_MENU, 0, 1},
-	{N_("Plugins and Scripts..."), menu_pluginlist, 0, M_MENU, 0, 1},
-	{N_("Raw Log..."), menu_rawlog, 0, M_MENU, 0, 1},	/* 51 */
-	{N_("URL Grabber..."), url_opengui, 0, M_MENU, 0, 1},
-	{0, 0, 0, M_SEP, 0, 0},
-	{N_("Reset Marker Line"), menu_resetmarker, 0, M_MENU, 0, 1, GDK_m},
-	{N_("C_lear Text"), menu_flushbuffer, GTK_STOCK_CLEAR, M_MENUSTOCK, 0, 1, GDK_l},
-	{N_("Search Text..."), menu_search, GTK_STOCK_FIND, M_MENUSTOCK, 0, 1, GDK_f},
-	{N_("Save Text..."), menu_savebuffer, GTK_STOCK_SAVE, M_MENUSTOCK, 0, 1},
+	{N_("_Window"), 0, 0, M_NEWMENU, 0, 0, 1},
+	{N_("Ban List..."), menu_banlist, 0, M_MENUITEM, 0, 0, 1},
+	{N_("Channel List..."), menu_chanlist, 0, M_MENUITEM, 0, 0, 1},
+	{N_("Character Chart..."), ascii_open, 0, M_MENUITEM, 0, 0, 1},
+	{N_("Direct Chat..."), menu_dcc_chat_win, 0, M_MENUITEM, 0, 0, 1},
+	{N_("File Transfers..."), menu_dcc_win, 0, M_MENUITEM, 0, 0, 1},
+	{N_("Ignore List..."), ignore_gui_open, 0, M_MENUITEM, 0, 0, 1},
+	{N_("Notify List..."), notify_opengui, 0, M_MENUITEM, 0, 0, 1},
+	{N_("Plugins and Scripts..."), menu_pluginlist, 0, M_MENUITEM, 0, 0, 1},
+	{N_("Raw Log..."), menu_rawlog, 0, M_MENUITEM, 0, 0, 1},	/* 60 */
+	{N_("URL Grabber..."), url_opengui, 0, M_MENUITEM, 0, 0, 1},
+	{0, 0, 0, M_SEP, 0, 0, 0},
+	{N_("Reset Marker Line"), menu_resetmarker, 0, M_MENUITEM, 0, 0, 1, GDK_m},
+	{N_("C_lear Text"), menu_flushbuffer, GTK_STOCK_CLEAR, M_MENUSTOCK, 0, 0, 1, GDK_l},
+	{N_("Search Text..."), menu_search, GTK_STOCK_FIND, M_MENUSTOCK, 0, 0, 1, GDK_f},
+	{N_("Save Text..."), menu_savebuffer, GTK_STOCK_SAVE, M_MENUSTOCK, 0, 0, 1},
 
-	{N_("_Help"), 0, 0, M_NEWMENU, 0, 1},	/* 62 */
-	{N_("_Contents"), menu_docs, GTK_STOCK_HELP, M_MENUSTOCK, 0, 1, GDK_F1},
-#ifdef GTK_STOCK_ABOUT
-	{N_("_About"), menu_about, GTK_STOCK_ABOUT, M_MENUSTOCK, 0, 1},
-#else
-	{N_("_About"), menu_about, (char *)&pix_about, M_MENUPIX, 0, 1},
-#endif
+	{N_("_Help"), 0, 0, M_NEWMENU, 0, 0, 1},	/* 67 */
+	{N_("_Contents"), menu_docs, GTK_STOCK_HELP, M_MENUSTOCK, 0, 0, 1, GDK_F1},
+	{N_("_About"), menu_about, GTK_STOCK_ABOUT, M_MENUSTOCK, 0, 0, 1},
 
-	{0, 0, 0, M_END, 0, 0},
+	{0, 0, 0, M_END, 0, 0, 0},
 };
 
 GtkWidget *
@@ -1603,7 +1730,7 @@ menu_add_plugin_items (GtkWidget *menu)
 
 GtkWidget *
 menu_create_main (void *accel_group, int bar, int away, int toplevel,
-						GtkWidget **away_item_ret, GtkWidget **user_menu_ret)
+						GtkWidget **menu_widgets)
 {
 	int i = 0;
 	GtkWidget *item;
@@ -1613,8 +1740,10 @@ menu_create_main (void *accel_group, int bar, int away, int toplevel,
 	GtkWidget *usermenu = 0;
 	GtkWidget *submenu = 0;
 	int close_mask = GDK_CONTROL_MASK;
+	int away_mask = GDK_MOD1_MASK;
 	char *key_theme = NULL;
 	GtkSettings *settings;
+	GSList *group = NULL;
 
 	if (bar)
 		menu_bar = gtk_menu_bar_new ();
@@ -1629,17 +1758,43 @@ menu_create_main (void *accel_group, int bar, int away, int toplevel,
 							G_CALLBACK (menu_canacaccel), 0);
 #endif
 
-	mymenu[19-menuoffset].state = prefs.invisible;
-	mymenu[20-menuoffset].state = prefs.wallops;
-	mymenu[21-menuoffset].state = prefs.servernotice;
-	mymenu[23-menuoffset].state = away;
-	mymenu[25-menuoffset].state = prefs.autorejoin;
-	mymenu[26-menuoffset].state = prefs.autoreconnect;
-	mymenu[27-menuoffset].state = prefs.autoreconnectonfail;
-	mymenu[29-menuoffset].state = prefs.autodialog;
-	mymenu[30-menuoffset].state = prefs.autodccchat;
-	mymenu[31-menuoffset].state = prefs.autodccsend;
-	/*mymenu[60-menuoffset].state = prefs.autosave;*/
+	/* set the initial state of toggles */
+	mymenu[MENUBAR_OFFSET].state = !prefs.hidemenu;
+	mymenu[MENUBAR_OFFSET+1].state = prefs.topicbar;
+	mymenu[MENUBAR_OFFSET+2].state = prefs.userlistbuttons;
+	mymenu[MENUBAR_OFFSET+3].state = prefs.chanmodebuttons;
+
+	mymenu[AWAY_OFFSET].state = away;
+
+	switch (prefs.tab_layout)
+	{
+	case 0:
+		mymenu[TABS_OFFSET].state = 1;
+		mymenu[TABS_OFFSET+1].state = 0;
+		break;
+	default:
+		mymenu[TABS_OFFSET].state = 0;
+		mymenu[TABS_OFFSET+1].state = 1;
+	}
+
+	mymenu[METRE_OFFSET].state = 0;
+	mymenu[METRE_OFFSET+1].state = 0;
+	mymenu[METRE_OFFSET+2].state = 0;
+	mymenu[METRE_OFFSET+3].state = 0;
+	switch (prefs.lagometer)
+	{
+	case 0:
+		mymenu[METRE_OFFSET].state = 1;
+		break;
+	case 1:
+		mymenu[METRE_OFFSET+1].state = 1;
+		break;
+	case 2:
+		mymenu[METRE_OFFSET+2].state = 1;
+		break;
+	default:
+		mymenu[METRE_OFFSET+3].state = 1;
+	}
 
 	/* change Close binding to ctrl-shift-w when using emacs keys */
 	settings = gtk_widget_get_settings (menu_bar);
@@ -1654,20 +1809,41 @@ menu_create_main (void *accel_group, int bar, int away, int toplevel,
 		}
 	}
 
+	/* Away binding to ctrl-alt-a if the _Help menu conflicts (FR/PT/IT) */
+	{
+		char *help = _("_Help");
+		char *under = strchr (help, '_');
+		if (under && (under[1] == 'a' || under[1] == 'A'))
+			away_mask = GDK_MOD1_MASK | GDK_CONTROL_MASK;
+	}
+
 	if (!toplevel)
-		mymenu[14-menuoffset].text = _("Detach Tab");
+	{
+		mymenu[DETACH_OFFSET].text = N_("_Detach Tab");
+		mymenu[CLOSE_OFFSET].text = N_("_Close Tab");
+	}
 	else
-		mymenu[14-menuoffset].text = _("Attach Tab");
+	{
+		mymenu[DETACH_OFFSET].text = N_("_Attach Window");
+		mymenu[CLOSE_OFFSET].text = N_("_Close Window");
+	}
 
 	while (1)
 	{
+		item = NULL;
+		if (mymenu[i].id == MENU_ID_USERMENU && !prefs.gui_usermenu)
+		{
+			i++;
+			continue;
+		}
+
 		switch (mymenu[i].type)
 		{
 		case M_NEWMENU:
 			if (menu)
 				gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
-			menu = gtk_menu_new ();
-			if (mymenu[i].callback == (void *) -1)
+			item = menu = gtk_menu_new ();
+			if (mymenu[i].id == MENU_ID_USERMENU)
 				usermenu = menu;
 			menu_item = gtk_menu_item_new_with_mnemonic (_(mymenu[i].text));
 			/* record the English name for /menu */
@@ -1684,7 +1860,7 @@ menu_create_main (void *accel_group, int bar, int away, int toplevel,
 			item = create_icon_menu (_(mymenu[i].text), mymenu[i].image, TRUE);
 			goto normalitem;
 
-		case M_MENU:
+		case M_MENUITEM:
 			item = gtk_menu_item_new_with_mnemonic (_(mymenu[i].text));
 normalitem:
 			if (mymenu[i].key != 0)
@@ -1702,26 +1878,35 @@ normalitem:
 			else
 				gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 			gtk_widget_show (item);
-			gtk_widget_set_sensitive (item, mymenu[i].activate);
 			break;
 
 		case M_MENUTOG:
-			item = gtk_check_menu_item_new_with_label (_(mymenu[i].text));
-			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
-													 mymenu[i].state);
+			item = gtk_check_menu_item_new_with_mnemonic (_(mymenu[i].text));
+togitem:
+			/* must avoid callback for Radio buttons */
+			GTK_CHECK_MENU_ITEM (item)->active = mymenu[i].state;
+			/*gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
+													 mymenu[i].state);*/
 			if (mymenu[i].key != 0)
 				gtk_widget_add_accelerator (item, "activate", accel_group,
-									mymenu[i].key, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
+									mymenu[i].key, mymenu[i].id == MENU_ID_AWAY ?
+									away_mask : GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
 			if (mymenu[i].callback)
 				g_signal_connect (G_OBJECT (item), "toggled",
 										G_CALLBACK (mymenu[i].callback), 0);
-			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+			if (submenu)
+				gtk_menu_shell_append (GTK_MENU_SHELL (submenu), item);
+			else
+				gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 			gtk_widget_show (item);
-			gtk_widget_set_sensitive (item, mymenu[i].activate);
-			if (bar && i == 23 - menuoffset)
-				*away_item_ret = item;
+			gtk_widget_set_sensitive (item, mymenu[i].sensitive);
 			break;
 
+		case M_MENURADIO:
+			item = gtk_radio_menu_item_new_with_mnemonic (group, _(mymenu[i].text));
+			group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (item));
+			goto togitem;
+
 		case M_SEP:
 			item = gtk_menu_item_new ();
 			gtk_widget_set_sensitive (item, FALSE);
@@ -1730,6 +1915,7 @@ normalitem:
 			break;
 
 		case M_MENUSUB:
+			group = NULL;
 			submenu = gtk_menu_new ();
 			item = create_icon_menu (_(mymenu[i].text), mymenu[i].image, TRUE);
 			/* record the English name for /menu */
@@ -1739,7 +1925,7 @@ normalitem:
 			gtk_widget_show (item);
 			break;
 
-		case M_END:
+		/*case M_END:*/ default:
 			if (!submenu)
 			{
 				if (menu)
@@ -1749,12 +1935,16 @@ normalitem:
 				}
 				if (usermenu)
 					usermenu_create (usermenu);
-				if (bar)
-					*user_menu_ret = usermenu;
 				return (menu_bar);
 			}
-			submenu = 0;
+			submenu = NULL;
 		}
+
+		/* record this GtkWidget * so it's state might be changed later */
+		if (mymenu[i].id != 0 && menu_widgets)
+			/* this ends up in sess->gui->menu_item[MENU_ID_XXX] */
+			menu_widgets[mymenu[i].id] = item;
+
 		i++;
 	}
 }
